︠e96377a9-5edf-4c4a-a69a-711020eab0d1si︠
%hide
%auto
def axes3d(x_range, y_range, z_range, **options):
    """Draw x, y, and z axes in 3D, and optionally a grid on the xy-plane

    'x_range' should have the form (x, xmin, xmax), and likewise for 'y_range'
    and 'z_range'. Here the 'x' can be either a symbolic variable, or a string.
    Either way, it is only used as a label for the corresponding axis.

    Options:
        'origin' - Locate the axes so that they all intersect at this point
                   (default (0,0,0))
        'grid' - Whether or not to draw a grid on the xy-plane (default True)
    All other options are as for the 'line3d' and 'arrow3d' commands, and are
    passed directly on to them.
    """

    x0, y0, z0 = options.pop("origin", (0, 0, 0))
    grid = options.pop("grid", True)
    gridoptions = options.copy()
    options.setdefault("color", "black")
    gridoptions.setdefault("color", "gray")
    x, xmin, xmax = x_range
    y, ymin, ymax = y_range
    z, zmin, zmax = z_range
    xmin, xmax = min(x0, xmin), max(x0, xmax)
    ymin, ymax = min(y0, ymin), max(y0, ymax)
    zmin, zmax = min(z0, zmin), max(z0, zmax)
    p = Graphics()
    p += arrow3d((xmin, y0, z0), (xmax, y0, z0), width=1, **options)
    p += arrow3d((x0, ymin, z0), (x0, ymax, z0), width=1, **options)
    p += arrow3d((x0, y0, zmin), (x0, y0, zmax), width=1, **options)
    p += text3d(str(x), (xmax + (xmax - xmin)*0.05, y0, z0), **options)
    p += text3d(str(y), (x0, ymax + (ymax - ymin)*0.05, z0), **options)
    p += text3d(str(z), (x0, y0, zmax + (zmax - zmin)*0.05), **options)
    xgridsize = 10**int(log(1.2*max(-xmin, xmax), 10))
    ygridsize = 10**int(log(1.2*max(-ymin, ymax), 10))
    zgridsize = 10**int(log(1.2*max(-zmin, zmax), 10))
    gridmin = ceil(xmin/xgridsize) * xgridsize
    for xval in srange(gridmin, xmax+xgridsize/100.0, xgridsize):
        #p += text3d("{:.0f}".format(float(xval)), (xval, ygridsize/5.0, 0), **options)
        if grid:
            p += line3d(((xval, ymin, 0), (xval, ymax, 0)), thickness=1, **gridoptions)
    gridmin = ceil(ymin/ygridsize) * ygridsize
    for yval in srange(gridmin, ymax+ygridsize/100.0, ygridsize):
        #p += text3d("{:.0f}".format(float(yval)), (xgridsize/5.0, yval, 0), **options)
        if grid:
            p += line3d(((xmin, yval, 0), (xmax, yval, 0)), thickness=1, **gridoptions)
    gridmin = ceil(zmin/zgridsize) * zgridsize
    #for zval in srange(gridmin, zmax+zgridsize/100.0, zgridsize):
        #p += text3d("{:.0f}".format(float(zval)), (xgridsize/5.0, 0, zval), **options)
    return p
︡cc46b156-7535-488e-8773-7cc0dfd986af︡{"hide":"input"}︡{"done":true}︡
︠596bd5f7-916f-4941-a820-6df7222f6481i︠
%hide
%md
#### Note: If you want to use any of the interactives in this worksheet (highly recommended), click the link that says “Open in CoCalc” at the top right of the page.

︡5c509ab0-7fe3-491f-8b6e-d44b27bc6b0f︡{"hide":"input"}︡{"md":"#### Note: If you want to use any of the interactives in this worksheet (highly recommended), click the link that says “Open in CoCalc” at the top right of the page. "}︡{"done":true}︡
︠5a10b7ce-2901-40ce-9829-fcba86af0011i︠
%hide
%md
# The period-doubling path to chaos
## and the corresponding bifurcation diagrams

︡8d5506bf-397c-4237-a221-6ef6e5c84ae4︡{"hide":"input"}︡{"md":"# The period-doubling path to chaos\n## and the corresponding bifurcation diagrams"}︡{"done":true}︡
︠f0f38953-8116-4a07-b485-f9a83a0aad15is︠
%hide
%md
We have seen in class that in the discrete-time logistic model,
$$ X_{t+1} = r \cdot X_t \cdot (1 - X_t) , $$
as we vary the parameter $r$ between $0$ and $4$, we get a series of bifurcations.

When $0 \le r < 3$, the system goes to a stable equilibrium. That equilibrium point is at $0$ when $r \le 1$, but is nonzero when $1 < r < 3$. Right at $r = 3$, a Hopf bifurcation occurs, so that when $r > 3$ (up to about $3.45$) the system goes to a stable oscillation between two $X$ values.

The interactive below allows you to explore this.
︡3f5892da-c042-44b2-af6c-9e4568dd1592︡{"hide":"input"}︡{"md":"We have seen in class that in the discrete-time logistic model, \n$$ X_{t+1} = r \\cdot X_t \\cdot (1 - X_t) , $$\nas we vary the parameter $r$ between $0$ and $4$, we get a series of bifurcations. \n\nWhen $0 \\le r < 3$, the system goes to a stable equilibrium. That equilibrium point is at $0$ when $r \\le 1$, but is nonzero when $1 < r < 3$. Right at $r = 3$, a Hopf bifurcation occurs, so that when $r > 3$ (up to about $3.45$) the system goes to a stable oscillation between two $X$ values. \n\nThe interactive below allows you to explore this. "}︡{"done":true}︡
︠67ddc3b1-8882-4e08-866e-50b8cf48e588s︠
@interact
def logistic(r=slider(0, RDF(3.43), RDF(0.01), default=2, label="$r$")):
    f(X) = r*X*(1 - X)
    solution = [0.1]
    for n in xrange(50):
        solution.append(f(solution[-1]))
    p = list_plot(solution, pointsize=30)
    p += list_plot(solution, plotjoined=True)
    p.show(ymin=0, ymax=1, axes_labels=("$t$", "$X$"))
︡74a2b2fe-ff90-43cd-abeb-8432b95e956f︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":200,"display_value":true,"label":"$r$","vals":["0.0","0.01","0.02","0.03","0.04","0.05","0.06","0.07","0.08","0.09","0.1","0.11","0.12","0.13","0.14","0.15","0.16","0.17","0.18","0.19","0.2","0.21","0.22","0.23","0.24","0.25","0.26","0.27","0.28","0.29","0.3","0.31","0.32","0.33","0.34","0.35","0.36","0.37","0.38","0.39","0.4","0.41","0.42","0.43","0.44","0.45","0.46","0.47","0.48","0.49","0.5","0.51","0.52","0.53","0.54","0.55","0.56","0.57","0.58","0.59","0.6","0.61","0.62","0.63","0.64","0.65","0.66","0.67","0.68","0.69","0.7","0.71","0.72","0.73","0.74","0.75","0.76","0.77","0.78","0.79","0.8","0.81","0.82","0.83","0.84","0.85","0.86","0.87","0.88","0.89","0.9","0.91","0.92","0.93","0.94","0.95","0.96","0.97","0.98","0.99","1.0","1.01","1.02","1.03","1.04","1.05","1.06","1.07","1.08","1.09","1.1","1.11","1.12","1.13","1.14","1.15","1.16","1.17","1.18","1.19","1.2","1.21","1.22","1.23","1.24","1.25","1.26","1.27","1.28","1.29","1.3","1.31","1.32","1.33","1.34","1.35","1.36","1.37","1.38","1.39","1.4","1.41","1.42","1.43","1.44","1.45","1.46","1.47","1.48","1.49","1.5","1.51","1.52","1.53","1.54","1.55","1.56","1.57","1.58","1.59","1.6","1.61","1.62","1.63","1.64","1.65","1.66","1.67","1.68","1.69","1.7","1.71","1.72","1.73","1.74","1.75","1.76","1.77","1.78","1.79","1.8","1.81","1.82","1.83","1.84","1.85","1.86","1.87","1.88","1.89","1.9","1.91","1.92","1.93","1.94","1.95","1.96","1.97","1.98","1.99","2.0","2.01","2.02","2.03","2.04","2.05","2.06","2.07","2.08","2.09","2.1","2.11","2.12","2.13","2.14","2.15","2.16","2.17","2.18","2.19","2.2","2.21","2.22","2.23","2.24","2.25","2.26","2.27","2.28","2.29","2.3","2.31","2.32","2.33","2.34","2.35","2.36","2.37","2.38","2.39","2.4","2.41","2.42","2.43","2.44","2.45","2.46","2.47","2.48","2.49","2.5","2.51","2.52","2.53","2.54","2.55","2.56","2.57","2.58","2.59","2.6","2.61","2.62","2.63","2.64","2.65","2.66","2.67","2.68","2.69","2.7","2.71","2.72","2.73","2.74","2.75","2.76","2.77","2.78","2.79","2.8","2.81","2.82","2.83","2.84","2.85","2.86","2.87","2.88","2.89","2.9","2.91","2.92","2.93","2.94","2.95","2.96","2.97","2.98","2.99","3.0","3.01","3.02","3.03","3.04","3.05","3.06","3.07","3.08","3.09","3.1","3.11","3.12","3.13","3.14","3.15","3.16","3.17","3.18","3.19","3.2","3.21","3.22","3.23","3.24","3.25","3.26","3.27","3.28","3.29","3.3","3.31","3.32","3.33","3.34","3.35","3.36","3.37","3.38","3.39","3.4","3.41","3.42","3.43"],"var":"r","width":null}],"flicker":false,"id":"475b5388-02bd-4daa-8c6a-2b53a7f05044","layout":[[["r",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠06698aa6-75de-4442-a88e-a3ec95b3cb3dis︠
%hide
%md
For $r$ values a little larger than $3.45$, the system starts to display a new type of behavior: after some transient behavior, it settles into a cycle that repeats itself, but rather than cycling between just two values of $X$, it cycles between ***four*** values! That is, instead of just repeating a pattern like A-B-A-B-A-B... (where A represents a low value of $X$ and B represents a higher value), it repeats a pattern of the form A-C-B-D-A-C-B-D-A-C-B-D. This is still periodic behavior, because it is repeating the same values over and over, but now the period is $4$ rather than $2$, because it takes $4$ time steps before it repeats.

Since increasing the parameter $r$ past the threshold of $3.45$ caused the oscillations to go from period $2$ to period $4$, we say that a ***period-doubling bifurcation*** occurred at $r \approx 3.45$.

This new “period $4$ behavior” can be seen in the steady-state part of the graph below.

︡89d3463c-cee1-4d8d-8a78-e9c062113746︡{"hide":"input"}︡{"md":"For $r$ values a little larger than $3.45$, the system starts to display a new type of behavior: after some transient behavior, it settles into a cycle that repeats itself, but rather than cycling between just two values of $X$, it cycles between ***four*** values! That is, instead of just repeating a pattern like A-B-A-B-A-B... (where A represents a low value of $X$ and B represents a higher value), it repeats a pattern of the form A-C-B-D-A-C-B-D-A-C-B-D. This is still periodic behavior, because it is repeating the same values over and over, but now the period is $4$ rather than $2$, because it takes $4$ time steps before it repeats.\n\nSince increasing the parameter $r$ past the threshold of $3.45$ caused the oscillations to go from period $2$ to period $4$, we say that a ***period-doubling bifurcation*** occurred at $r \\approx 3.45$.\n\nThis new “period $4$ behavior” can be seen in the steady-state part of the graph below."}︡{"done":true}︡
︠d1483855-7e53-40c1-9599-7d65a2604d4c︠
r = 3.5
f(X) = r*X*(1 - X)
solution = [0.1]
for n in xrange(50):
    solution.append(f(solution[-1]))
p = list_plot(solution, pointsize=30)
p += list_plot(solution, plotjoined=True)
p += text("$r = {}$".format(RDF(r)), (25, 1), color="black", fontsize=16)
p.show(ymin=0, ymax=1, axes_labels=("$t$", "$X$"))
︡17a0501b-66d5-4509-b086-6367807c2aeb︡{"file":{"filename":"/home/user/.sage/temp/project-cdb600df-721d-4ab1-aa2a-b8fb0f7750b8/195/tmp_b0PYFU.svg","show":true,"text":null,"uuid":"dfcade7b-e6d9-410e-9819-0d99ff325252"},"once":false}︡{"done":true}︡
︠bc41cf62-d76e-432b-9e23-b4ff0f89743bis︠
%hide
%md
Slightly farther along, after about $r = 3.54$, the pattern of behavior changes again, this time to A-E-D-G-B-F-C-H-A-E-D-G-B-F-C-H-A-E-D-G-B-F-C-H. That is, the cycle now has period $8$. Another period-doubling bifurcation has occurred!

After around $r = 3.56$, the behavior changes to period $16$ (another period-doubling bifurcation), and after about $r = 3.565$, it changes to period $32$ (yet another period-doubling bifurcation). You can explore all of these in the interactive below.

What you should be noticing about the $r$ values mentioned above is that they are getting closer and closer together. Here is a summary:
 - $r =       3.00$: Hopf bifurcation, start of period 2 oscillations
 - $r \approx 3.45$: First period-doubling bifurcation, start of period 4 oscillations
 - $r \approx 3.543$: Second period-doubling bifurcation, start of period 8 oscillations
 - $r \approx 3.562$: Third period-doubling bifurcation, start of period 16 oscillations
 - $r \approx 3.5665$: Fourth period-doubling bifurcation, start of period 32 oscillations
 - etc...

In fact, between the Hopf bifurcation at $r = 3$ and somewhere around $r = 3.56995$, an *infinite* number of period-doubling bifurcations occur! But because they keep occuring closer and closer together, all infinitely many of them fit within the relatively short interval of $r$ values from $3.45$ to $3.56995$.

Then, for values of $r$ greater than about $3.56995$, the behavior changes again, this time to ***chaotic behavior***.

In the interactive below, you can see the behavior for specific $r$ values in this range, plus a few larger $r$ values showing chaotic behavior. The first plot shows the time series from $t = 200$ to $t = 250$. The first $200$ iterations have been discarded, to get rid of any transient behavior and just show the steady-state pattern. Although it's a discrete-time model, this plot was made with `plotjoined=True` to help show the pattern of the oscillation. In the same interactive, the second plot also shows only the steady-state behavior (by discarding the first $200$ iterations again), but this time shows many more iterations, with `plotjoined=False` and very small points. This makes it much easier to see how many distinct $X$ values are hit in the oscillation pattern, so it's easy to see what the period of the oscillation is.

︡95d7da35-a17f-4014-b636-785b5eb91417︡{"hide":"input"}︡{"md":"Slightly farther along, after about $r = 3.54$, the pattern of behavior changes again, this time to A-E-D-G-B-F-C-H-A-E-D-G-B-F-C-H-A-E-D-G-B-F-C-H. That is, the cycle now has period $8$. Another period-doubling bifurcation has occurred!\n\nAfter around $r = 3.56$, the behavior changes to period $16$ (another period-doubling bifurcation), and after about $r = 3.565$, it changes to period $32$ (yet another period-doubling bifurcation). You can explore all of these in the interactive below.\n\nWhat you should be noticing about the $r$ values mentioned above is that they are getting closer and closer together. Here is a summary:\n - $r =       3.00$: Hopf bifurcation, start of period 2 oscillations\n - $r \\approx 3.45$: First period-doubling bifurcation, start of period 4 oscillations\n - $r \\approx 3.543$: Second period-doubling bifurcation, start of period 8 oscillations\n - $r \\approx 3.562$: Third period-doubling bifurcation, start of period 16 oscillations\n - $r \\approx 3.5665$: Fourth period-doubling bifurcation, start of period 32 oscillations\n - etc...\n\nIn fact, between the Hopf bifurcation at $r = 3$ and somewhere around $r = 3.56995$, an *infinite* number of period-doubling bifurcations occur! But because they keep occuring closer and closer together, all infinitely many of them fit within the relatively short interval of $r$ values from $3.45$ to $3.56995$.\n\nThen, for values of $r$ greater than about $3.56995$, the behavior changes again, this time to ***chaotic behavior***.\n\nIn the interactive below, you can see the behavior for specific $r$ values in this range, plus a few larger $r$ values showing chaotic behavior. The first plot shows the time series from $t = 200$ to $t = 250$. The first $200$ iterations have been discarded, to get rid of any transient behavior and just show the steady-state pattern. Although it's a discrete-time model, this plot was made with `plotjoined=True` to help show the pattern of the oscillation. In the same interactive, the second plot also shows only the steady-state behavior (by discarding the first $200$ iterations again), but this time shows many more iterations, with `plotjoined=False` and very small points. This makes it much easier to see how many distinct $X$ values are hit in the oscillation pattern, so it's easy to see what the period of the oscillation is. "}︡{"done":true}︡
︠837981fb-2e8f-45eb-83c2-eac578fbb304s︠
@interact
def logistic(r=slider([3.4, 3.5, 3.56, 3.566, 3.575, 3.6, 4])):
    f(X) = r*X*(1 - X)
    solution = [0.1]
    for t in xrange(1, 2000):
        solution.append(f(solution[-1]))
    p = list_plot(zip(range(200, 250), solution[200:250]), plotjoined=True)
    p.show(axes_labels=("$t$", "$X$"), ymin=0, ymax=1)
    p = list_plot(zip(range(200, 2000), solution[200:2000]), pointsize=1)
    p.show(axes_labels=("$t$", "$X$"), ymin=0, ymax=1)
︡a6cf512b-3cb7-40b4-8894-1ca8919be0ad︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"r","vals":["3.40000000000000","3.50000000000000","3.56000000000000","3.56600000000000","3.57500000000000","3.60000000000000","4"],"var":"r","width":null}],"flicker":false,"id":"e4aa883e-fb7d-4137-b713-5ac5acab3968","layout":[[["r",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠5a7d4a4d-796f-41e9-8eb9-0ad4246f54bbis︠
%hide
%md
We can create a bifurcation diagram that summarizes all of these behaviors in a way that's very similar to the bifurcation diagrams we studied in LS 30A. The parameter ($r$) value goes on the horizontal axis, and the vertical axis is the state variable ($X$), so that each vertical slice of the graph represents a copy of the state space for a specific $r$ value. However, instead of just plotting the equilibrium points (and indicating their stability with colors or a line pattern), here we will plot, for each $r$ value, all of the $X$ values that are part of the steady-state oscillation pattern at that value of $r$. So counting the points in any vertical slice of the diagram will tell you the period of the oscillation. Here is that diagram:

<img src="BifurcationDiagram_Logistic_Full.png" alt="Complete bifurcation diagram of the discrete-time logistic model" width="780" height="484">

As you can see, the Hopf bifurcation at $r = 3$ shows up here as a “branching”, where the curve forks into two curves. Likewise, each period-doubling bifurcation appears as an $r$ value at which all of the curves simultaneously branch in the same way: $2$ curves branch into $4$ curves, then $4$ curves branch into $8$ curves, then $8$ curves branch into $16$ curves, etc.

The regions of chaotic behavior (most of the graph where $r$ is greater than about $3.6$) show up as a sort of fuzzy gray area, because for these $r$ values, the system no longer generates a finite sequence of $X$ values that repeats periodically. Instead, since chaotic behavior is *aperiodic*, the sequence of states will include infinitely many different $X$ values. To produce this picture, instead of just plotting a point for every $X$ value, we have shaded each point lighter or darker depending on how frequently the sequence of $X$ values hit that point. When the behavior is periodic, every $X$ value in the periodic pattern gets hit just as often as every other one, so those points are colored uniformly black. But when it's chaotic, some $X$ values occur more frequently than others, so some are shaded darker and some are lighter.

(Note: I have not included the code used to produce these diagrams in this worksheet, but they were produced in CoCalc using SageMath. If you'd like to see/use the code yourself, contact me.)

Here is a higher-resolution version, showing just the more interesting $r$ range from $2.5$ to $4$.

<img src="BifurcationDiagram_Logistic_HiRes.png" alt="Hi-res bifurcation diagram of the discrete-time logistic model, 2.5 ≤ r ≤ 4" width="925" height="605">

One thing that you might notice in these diagrams is that, for some $r$ values greater than $3.6$, the chaotic behavior seems to suddenly disappear for a little while, and a few individual curves appear again, indicating that the system has returned to periodic behavior! Probably the most visible of these regions is around $r = 3.83$. What does the period seem to be there? If you look closely, shortly after that $r$ value, the system goes through another period-doubling bifurcation. In fact, although it's hard to see it without zooming in much more, the system will once again undergo *infinitely many more* period-doubling bifurcations (all between $r = 3.83$ and about $r = 3.85$) before returning to chaotic behavior! This means that if you were to zoom in on a rectangle defined by roughly $3.83 \le r \le 3.85$ and $0.9 \le X \le 1$, the diagram would look very similar to the region of the graph from $r = 2.5$ to $r = 3.6$... that is, most of the whole graph that you see above! Mathematicians call this property *self-similarity*, and images that are self-similar are often referred to as *fractals*. The self-similarity of bifurcation diagrams like this one is one of the deep connections between chaotic dynamical systems and fractal geometry.

The region we've just described isn't the only one with periodic behavior in the middle of all the chaos. Everywhere in the chaotic region where you see a white vertical band, there should be periodic behavior with some finite period. With the help of the diagram above, see if you can use the interactive below to find $r$ values where the behavior is periodic with period $3$, period $6$, period $5$, and period $7$. What other periods can you find?

︡140d8fd9-f69b-45e4-9088-56d09856663d︡{"hide":"input"}︡{"md":"We can create a bifurcation diagram that summarizes all of these behaviors in a way that's very similar to the bifurcation diagrams we studied in LS 30A. The parameter ($r$) value goes on the horizontal axis, and the vertical axis is the state variable ($X$), so that each vertical slice of the graph represents a copy of the state space for a specific $r$ value. However, instead of just plotting the equilibrium points (and indicating their stability with colors or a line pattern), here we will plot, for each $r$ value, all of the $X$ values that are part of the steady-state oscillation pattern at that value of $r$. So counting the points in any vertical slice of the diagram will tell you the period of the oscillation. Here is that diagram:\n\n<img src=\"BifurcationDiagram_Logistic_Full.png\" alt=\"Complete bifurcation diagram of the discrete-time logistic model\" width=\"780\" height=\"484\">\n\nAs you can see, the Hopf bifurcation at $r = 3$ shows up here as a “branching”, where the curve forks into two curves. Likewise, each period-doubling bifurcation appears as an $r$ value at which all of the curves simultaneously branch in the same way: $2$ curves branch into $4$ curves, then $4$ curves branch into $8$ curves, then $8$ curves branch into $16$ curves, etc.\n\nThe regions of chaotic behavior (most of the graph where $r$ is greater than about $3.6$) show up as a sort of fuzzy gray area, because for these $r$ values, the system no longer generates a finite sequence of $X$ values that repeats periodically. Instead, since chaotic behavior is *aperiodic*, the sequence of states will include infinitely many different $X$ values. To produce this picture, instead of just plotting a point for every $X$ value, we have shaded each point lighter or darker depending on how frequently the sequence of $X$ values hit that point. When the behavior is periodic, every $X$ value in the periodic pattern gets hit just as often as every other one, so those points are colored uniformly black. But when it's chaotic, some $X$ values occur more frequently than others, so some are shaded darker and some are lighter.\n\n(Note: I have not included the code used to produce these diagrams in this worksheet, but they were produced in CoCalc using SageMath. If you'd like to see/use the code yourself, contact me.)\n\nHere is a higher-resolution version, showing just the more interesting $r$ range from $2.5$ to $4$.\n\n<img src=\"BifurcationDiagram_Logistic_HiRes.png\" alt=\"Hi-res bifurcation diagram of the discrete-time logistic model, 2.5 ≤ r ≤ 4\" width=\"925\" height=\"605\">\n\nOne thing that you might notice in these diagrams is that, for some $r$ values greater than $3.6$, the chaotic behavior seems to suddenly disappear for a little while, and a few individual curves appear again, indicating that the system has returned to periodic behavior! Probably the most visible of these regions is around $r = 3.83$. What does the period seem to be there? If you look closely, shortly after that $r$ value, the system goes through another period-doubling bifurcation. In fact, although it's hard to see it without zooming in much more, the system will once again undergo *infinitely many more* period-doubling bifurcations (all between $r = 3.83$ and about $r = 3.85$) before returning to chaotic behavior! This means that if you were to zoom in on a rectangle defined by roughly $3.83 \\le r \\le 3.85$ and $0.9 \\le X \\le 1$, the diagram would look very similar to the region of the graph from $r = 2.5$ to $r = 3.6$... that is, most of the whole graph that you see above! Mathematicians call this property *self-similarity*, and images that are self-similar are often referred to as *fractals*. The self-similarity of bifurcation diagrams like this one is one of the deep connections between chaotic dynamical systems and fractal geometry.\n\nThe region we've just described isn't the only one with periodic behavior in the middle of all the chaos. Everywhere in the chaotic region where you see a white vertical band, there should be periodic behavior with some finite period. With the help of the diagram above, see if you can use the interactive below to find $r$ values where the behavior is periodic with period $3$, period $6$, period $5$, and period $7$. What other periods can you find?"}︡{"done":true}︡
︠df5a6fbf-1bb9-4b1a-b82d-fa42f740ad78s︠
@interact
def logistic(r=slider(RDF(3.569), 4, RDF(0.001), label="$r$")):
    f(X) = r*X*(1 - X)
    solution = [0.1]
    for t in xrange(1, 2000):
        solution.append(f(solution[-1]))
    p = list_plot(zip(range(200, 250), solution[200:250]), plotjoined=True)
    p.show(axes_labels=("$t$", "$X$"), ymin=0, ymax=1)
    p = list_plot(zip(range(200, 2000), solution[200:2000]), pointsize=1)
    p.show(axes_labels=("$t$", "$X$"), ymin=0, ymax=1)
︡735595a9-5a4c-4a23-8bcc-d9a79475e797︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"$r$","vals":["3.569","3.57","3.571","3.572","3.573","3.574","3.575","3.576","3.577","3.578","3.579","3.58","3.581","3.582","3.583","3.584","3.585","3.586","3.587","3.588","3.589","3.59","3.591","3.592","3.593","3.594","3.595","3.596","3.597","3.598","3.599","3.6","3.601","3.602","3.603","3.604","3.605","3.606","3.607","3.608","3.609","3.61","3.611","3.612","3.613","3.614","3.615","3.616","3.617","3.618","3.619","3.62","3.621","3.622","3.623","3.624","3.625","3.626","3.627","3.628","3.629","3.63","3.631","3.632","3.633","3.634","3.635","3.636","3.637","3.638","3.639","3.64","3.641","3.642","3.643","3.644","3.645","3.646","3.647","3.648","3.649","3.65","3.651","3.652","3.653","3.654","3.655","3.656","3.657","3.658","3.659","3.66","3.661","3.662","3.663","3.664","3.665","3.666","3.667","3.668","3.669","3.67","3.671","3.672","3.673","3.674","3.675","3.676","3.677","3.678","3.679","3.68","3.681","3.682","3.683","3.684","3.685","3.686","3.687","3.688","3.689","3.69","3.691","3.692","3.693","3.694","3.695","3.696","3.697","3.698","3.699","3.7","3.701","3.702","3.703","3.704","3.705","3.706","3.707","3.708","3.709","3.71","3.711","3.712","3.713","3.714","3.715","3.716","3.717","3.718","3.719","3.72","3.721","3.722","3.723","3.724","3.725","3.726","3.727","3.728","3.729","3.73","3.731","3.732","3.733","3.734","3.735","3.736","3.737","3.738","3.739","3.74","3.741","3.742","3.743","3.744","3.745","3.746","3.747","3.748","3.749","3.75","3.751","3.752","3.753","3.754","3.755","3.756","3.757","3.758","3.759","3.76","3.761","3.762","3.763","3.764","3.765","3.766","3.767","3.768","3.769","3.77","3.771","3.772","3.773","3.774","3.775","3.776","3.777","3.778","3.779","3.78","3.781","3.782","3.783","3.784","3.785","3.786","3.787","3.788","3.789","3.79","3.791","3.792","3.793","3.794","3.795","3.796","3.797","3.798","3.799","3.8","3.801","3.802","3.803","3.804","3.805","3.806","3.807","3.808","3.809","3.81","3.811","3.812","3.813","3.814","3.815","3.816","3.817","3.818","3.819","3.82","3.821","3.822","3.823","3.824","3.825","3.826","3.827","3.828","3.829","3.83","3.831","3.832","3.833","3.834","3.835","3.836","3.837","3.838","3.839","3.84","3.841","3.842","3.843","3.844","3.845","3.846","3.847","3.848","3.849","3.85","3.851","3.852","3.853","3.854","3.855","3.856","3.857","3.858","3.859","3.86","3.861","3.862","3.863","3.864","3.865","3.866","3.867","3.868","3.869","3.87","3.871","3.872","3.873","3.874","3.875","3.876","3.877","3.878","3.879","3.88","3.881","3.882","3.883","3.884","3.885","3.886","3.887","3.888","3.889","3.89","3.891","3.892","3.893","3.894","3.895","3.896","3.897","3.898","3.899","3.9","3.901","3.902","3.903","3.904","3.905","3.906","3.907","3.908","3.909","3.91","3.911","3.912","3.913","3.914","3.915","3.916","3.917","3.918","3.919","3.92","3.921","3.922","3.923","3.924","3.925","3.926","3.927","3.928","3.929","3.93","3.931","3.932","3.933","3.934","3.935","3.936","3.937","3.938","3.939","3.94","3.941","3.942","3.943","3.944","3.945","3.946","3.947","3.948","3.949","3.95","3.951","3.952","3.953","3.954","3.955","3.956","3.957","3.958","3.959","3.96","3.961","3.962","3.963","3.964","3.965","3.966","3.967","3.968","3.969","3.97","3.971","3.972","3.973","3.974","3.975","3.976","3.977","3.978","3.979","3.98","3.981","3.982","3.983","3.984","3.985","3.986","3.987","3.988","3.989","3.99","3.991","3.992","3.993","3.994","3.995","3.996","3.997","3.998","3.999","4.0"],"var":"r","width":null}],"flicker":false,"id":"0f5bbc3b-d6d7-4f6e-b0d9-e01e8b5a1478","layout":[[["r",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠bc812781-47c8-405c-8fb2-23f0020ab41dis︠
%hide
%md
## What about other chaotic systems?

Does a similar pattern of bifurcations happen in other models that show chaotic behavior? And in particular, can we find anything equivalent to the period-doubling bifurcations that we saw above in a continuous-time (differential equation) model? Could we even make a bifurcation diagram in a similar way, and if so, what will it look like?

The answers to all of these questions are yes, yes, yes, and surprisingly similar to the bifurcation diagrams that we just saw!

My favorite example of this in continuous time is the Rössler model (which you covered in one of your labs as the “Romeo-Juliet-Tybalt” model). As a reminder, in this model, $R$ represents Romeo's love (or hatred if negative) for Juliet, $J$ represents Juliet's love/hatred for Romeo, and $T$ represents Tybalt's *hatred* of Romeo. The system of differential equations is
$$ \begin{cases}
    R' &= -J - T \\
    J' &= R + 0.1 J \\
    T' &= 0.1 - a T + R T
\end{cases} $$
In your lab, you studied this model with the parameter $a$ (in the $T'$ equation) set to $14$. But since we're interested in studying bifurcations of this model, we need to vary a parameter, so the parameter $a$ is the one we'll work with.

Note that since there are three state variables in this model (as is necessary to get chaotic behavior from an ODE), the graphs of trajectories shown below will all be 3D graphs.

The first graph below shows, in different colors, a few different trajectories of this model for the parameter value $a = 2$. You should see a very familiar feature here.

︡6cab2fca-836b-4a2b-a3b5-ffd7527999cc︡{"hide":"input"}︡{"md":"## What about other chaotic systems?\n\nDoes a similar pattern of bifurcations happen in other models that show chaotic behavior? And in particular, can we find anything equivalent to the period-doubling bifurcations that we saw above in a continuous-time (differential equation) model? Could we even make a bifurcation diagram in a similar way, and if so, what will it look like?\n\nThe answers to all of these questions are yes, yes, yes, and surprisingly similar to the bifurcation diagrams that we just saw!\n\nMy favorite example of this in continuous time is the Rössler model (which you covered in one of your labs as the “Romeo-Juliet-Tybalt” model). As a reminder, in this model, $R$ represents Romeo's love (or hatred if negative) for Juliet, $J$ represents Juliet's love/hatred for Romeo, and $T$ represents Tybalt's *hatred* of Romeo. The system of differential equations is\n$$ \\begin{cases}\n    R' &= -J - T \\\\\n    J' &= R + 0.1 J \\\\\n    T' &= 0.1 - a T + R T\n\\end{cases} $$\nIn your lab, you studied this model with the parameter $a$ (in the $T'$ equation) set to $14$. But since we're interested in studying bifurcations of this model, we need to vary a parameter, so the parameter $a$ is the one we'll work with.\n\nNote that since there are three state variables in this model (as is necessary to get chaotic behavior from an ODE), the graphs of trajectories shown below will all be 3D graphs.\n\nThe first graph below shows, in different colors, a few different trajectories of this model for the parameter value $a = 2$. You should see a very familiar feature here.\n\n"}︡{"done":true}︡
︠b8f17e4a-7ea6-4882-8252-f3ec1a8b997cs︠
a = 2
vectorfield(R, J, T) = (-J - T, R + 0.1*J, 0.1 - a*T + R*T)
t_range = srange(0, 100, 0.01)
p = axes3d((R, -5, 5), (J, -5, 5), (T, 0, 5), grid=False)
solution = desolve_odeint(vectorfield, (0, 4, 0), t_range, [R, J, T])
p += list_plot(solution, plotjoined=True, color="red")
solution = desolve_odeint(vectorfield, (1, 1, 0), t_range, [R, J, T])
p += list_plot(solution, plotjoined=True, color="fuchsia")
solution = desolve_odeint(vectorfield, (0, -3.78, 0.034), t_range, [R, J, T])
p += list_plot(solution, plotjoined=True, color="blue", thickness=2)
p.show(aspect_ratio=(1, 1, 1), frame=False)
︡3dc3b7b6-96c9-4af8-8ab0-ae4c7c4548a4︡{"file":{"filename":"c9def685-4b22-4143-9020-98b2bea16bd1.sage3d","uuid":"c9def685-4b22-4143-9020-98b2bea16bd1"}}︡{"done":true}︡
︠13a9f0c3-06d7-4d29-94e5-ec1f91ea92acis︠
%hide
%md
As you can see, the slightly thicker blue loop, which the red and pink trajectories approach asymptotically, is a **limit cycle attractor**. So when $a = 2$, this model already shows periodic behavior. (It turns out that it's not possible to get a stable equilibrium point for any value of the parameter $a$, so we can't get a Hopf bifurcation by varying this parameter.)

From now on, since we are only interested in the steady-state behavior, we will do as we did for the discrete-time logistic model above, and throw away the earlier part of our simulation to get rid of any transients.

The next two graphs show just the limit cycle attractor for the parameter values $a = 4$ and $a = 6$. What happens between these two values?

︡0bcef9f0-a619-4f89-b94a-d41614ead495︡{"hide":"input"}︡{"md":"As you can see, the slightly thicker blue loop, which the red and pink trajectories approach asymptotically, is a **limit cycle attractor**. So when $a = 2$, this model already shows periodic behavior. (It turns out that it's not possible to get a stable equilibrium point for any value of the parameter $a$, so we can't get a Hopf bifurcation by varying this parameter.)\n\nFrom now on, since we are only interested in the steady-state behavior, we will do as we did for the discrete-time logistic model above, and throw away the earlier part of our simulation to get rid of any transients. \n\nThe next two graphs show just the limit cycle attractor for the parameter values $a = 4$ and $a = 6$. What happens between these two values? "}︡{"done":true}︡
︠1037b2be-d53a-4541-94fe-6ffe79215447s︠
t_range = srange(0, 100, 0.01)
a = 4
vectorfield1(R, J, T) = (-J - T, R + 0.1*J, 0.1 - a*T + R*T)
solution1 = desolve_odeint(vectorfield1, (1, 1, 0), t_range, [R, J, T])
a = 6
vectorfield2(R, J, T) = (-J - T, R + 0.1*J, 0.1 - a*T + R*T)
solution2 = desolve_odeint(vectorfield2, (1, 1, 0), t_range, [R, J, T])

start = int(75 / 0.01)
show("$\Large a = 4$")
p = axes3d((R, -12, 12), (J, -12, 12), (T, 0, 5), grid=False)
p += list_plot(solution1[start:], plotjoined=True, color="blue")
p.show(aspect_ratio=(1, 1, 1), frame=False)

print("\n\n\n")

show("$\Large a = 6$")
p = axes3d((R, -12, 12), (J, -12, 12), (T, 0, 5), grid=False)
p += list_plot(solution2[start:], plotjoined=True, color="blue")
p.show(aspect_ratio=(1, 1, 1), frame=False)
︡494e15c9-a74c-4173-8832-c8dbb9172536︡{"html":"<div align='center'>$\\Large a = 4$</div>"}︡{"file":{"filename":"fc28b15c-2f89-4252-b4e1-754dcf8b68b5.sage3d","uuid":"fc28b15c-2f89-4252-b4e1-754dcf8b68b5"}}︡{"stdout":"\n\n\n\n"}︡{"html":"<div align='center'>$\\Large a = 6$</div>"}︡{"file":{"filename":"8eb22899-c8f9-4ea9-aaca-a7ec6ca74a2d.sage3d","uuid":"8eb22899-c8f9-4ea9-aaca-a7ec6ca74a2d"}}︡{"done":true}︡
︠d95ea700-1303-465b-bef9-df5561bcf33cis︠
%hide
%md
### What just happened?

If you look closely, each of the above trajectories is a closed loop, so the behavior we're seeing here is periodic. However, in the second one, the trajectory has wound around the $T$ axis twice before closing the loop. If you consider the amount of time required for the trajectory to wind around the $T$ axis once, it's not hard to imagine (even though we can't see $t$ in the graph) that it would take about twice as much time to go around twice. In other words, it's likely the the *period* of the oscillatory behavior in the second ($a = 6$) graph is about twice as long as the period of the oscillation in the first ($a = 4$) graph.

To summarize what we just said: as the parameter $a$ was increased, somewhere between $a = 4$ and $a = 6$ the period of the oscillation doubled. What does that mean? You guessed it. That's a period-doubling bifurcation!

To see what we mean about the period as a time interval, take a look at the two time series below for the $T$ variable. The period of the oscillation can be measured here as the horizontal distance between the peaks. In the first ($a = 4$) graph, that period is approximately $6$. In the second ($a = 6$) graph, it's approximately $12$. Hence, the period doubled.

︡d44b5de8-2f40-4eb7-b6e5-ba525ac01368︡{"hide":"input"}︡{"md":"### What just happened? \n\nIf you look closely, each of the above trajectories is a closed loop, so the behavior we're seeing here is periodic. However, in the second one, the trajectory has wound around the $T$ axis twice before closing the loop. If you consider the amount of time required for the trajectory to wind around the $T$ axis once, it's not hard to imagine (even though we can't see $t$ in the graph) that it would take about twice as much time to go around twice. In other words, it's likely the the *period* of the oscillatory behavior in the second ($a = 6$) graph is about twice as long as the period of the oscillation in the first ($a = 4$) graph. \n\nTo summarize what we just said: as the parameter $a$ was increased, somewhere between $a = 4$ and $a = 6$ the period of the oscillation doubled. What does that mean? You guessed it. That's a period-doubling bifurcation! \n\nTo see what we mean about the period as a time interval, take a look at the two time series below for the $T$ variable. The period of the oscillation can be measured here as the horizontal distance between the peaks. In the first ($a = 4$) graph, that period is approximately $6$. In the second ($a = 6$) graph, it's approximately $12$. Hence, the period doubled. "}︡{"done":true}︡
︠c08a621d-92e5-4ba5-be32-20256224b208s︠
p = list_plot(zip(t_range, solution1[:,2])[start:], plotjoined=True)
p += text("$a = 4$", (87.5, 9), color="black", fontsize=16)
p.show(ymax=10, axes_labels=("$t$", "$T$"))

p = list_plot(zip(t_range, solution2[:,2])[start:], plotjoined=True)
p += text("$a = 6$", (87.5, 9), color="black", fontsize=16)
p.show(ymax=10, axes_labels=("$t$", "$T$"))
︡d4fbf8f0-c6cc-4c45-b913-b49f4025f4d3︡{"file":{"filename":"/home/user/.sage/temp/project-cdb600df-721d-4ab1-aa2a-b8fb0f7750b8/339/tmp_0rZkUe.svg","show":true,"text":null,"uuid":"94bcb44f-2fbe-4446-bf49-57bb9e30ba2b"},"once":false}︡{"file":{"filename":"/home/user/.sage/temp/project-cdb600df-721d-4ab1-aa2a-b8fb0f7750b8/339/tmp_oXX07K.svg","show":true,"text":null,"uuid":"684e13de-8d34-44f0-93ce-6504a27364d1"},"once":false}︡{"done":true}︡
︠ad8d170a-68d2-418d-9d04-42dff61fe4a2is︠
%hide
%md
The interactive below will show you both the trajectory and the time series (for the $T$ variable) of this model, for a handful of specific values of $a$. These values of $a$ show a sequence of period-doubling bifurcations, and eventually chaotic behavior.

︡a0197814-9f06-4fce-90da-74208fca1edc︡{"hide":"input"}︡{"md":"The interactive below will show you both the trajectory and the time series (for the $T$ variable) of this model, for a handful of specific values of $a$. These values of $a$ show a sequence of period-doubling bifurcations, and eventually chaotic behavior. "}︡{"done":true}︡
︠5c720d67-e85c-486a-8e6d-29f465822ea0s︠
@interact
def rossler(a=slider([4, 6, 8.5, 8.7, 8.73, 8.74, 8.75, 8.76, 9], label="$a$")):
    vectorfield(R, J, T) = (-J - T, R + 0.1*J, 0.1 - a*T + R*T)
    t_range = srange(0, 1000 if a < 8.74 else 1500, 0.01)
    start = int(900/0.01)
    solution = desolve_odeint(vectorfield, (1, 1, 0), t_range, [R, J, T])
    p = axes3d((R, -20, 20), (J, -20, 20), (T, 0, 20), grid=False)
    p += list_plot(solution[start:], plotjoined=True, color="blue")
    p.show(aspect_ratio=(1, 1, 1), frame=False)
    p2 = list_plot(zip(t_range, solution[:,2])[start:], plotjoined=True)
    p2.show(ymax=20, axes_labels=("$t$", "$T$"))
︡e60cf3be-f7dc-4bb9-8fe6-c59a31ccce54︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"$a$","vals":["4","6","8.50000000000000","8.70000000000000","8.73000000000000","8.74000000000000","8.75000000000000","8.76000000000000","9"],"var":"a","width":null}],"flicker":false,"id":"bd484538-a060-4d9b-badc-e70c64370d8d","layout":[[["a",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠331cf577-f99a-4ab8-ab14-1e1c8f09c51ais︠
%hide
%md
As with the discrete-time logistic model that we considered earlier, we can create a bifurcation diagram showing the changes in behavior as we vary the parameter $a$. Here it is:

<img src="BifurcationDiagram_Rossler.png" alt="Bifurcation diagram of the Rossler model" width="937" height="580">

Notice how similar this is to the bifurcation diagram from the discrete-time logistic model. Of course, the precise details are different. But the overall shape or pattern of behavior is exactly the same: we see a sequence of period-doubling bifurcations, occurring closer and closer together, until suddenly chaotic behavior happens. Then, within the region of chaotic behavior, there are small windows of periodic behavior that appear for short parameter intervals, and even within these more period-doubling bifurcations can be seen. Also, within the region of chaotic behavior, some values of the state variable ($J$ in this case) occur more frequently, and others less frequently, resulting in darker or lighter shading. From this, a complex pattern can be seen through the chaos.

If you're following closely how these bifurcation diagrams are created, you might be wondering how this one can be made, since there are now $3$ state variables in this model instead of $1$, and the trajectory moves smoothly through the state space instead of jumping from one point to another as in a discrete-time model. In this case, we have “discretized” the steady-state behavior of the system by looking at the points where the trajectory intersects the $JT$ plane. We have then decreased these $3$-dimensional points to $1$-dimensional data by looking only at the value of the $J$ variable at each point (since the $T$ values at these points are quite close to $0$ and don't vary much). This is the reason that the vertical axis in the bifurcation diagram above is labeled $J$: it gives the $J$ coordinates of the points at which the trajectory intersects the $JT$ plane. Similar tricks can be employed in other continuous-time models with chaotic behavior.

You can use the interactive below to explore this model further for the whole range of values of $a$ shown above. With the help of the above bifurcation diagram, see if you can use this interactive to find $a$ values where the limit cycle attractor winds around the $T$ axis $3$ times before forming a loop. What about $6$ times, or $5$ times, or $7$ times?

︡1b366355-6890-4c76-80a0-cc3f9f398e13︡{"hide":"input"}︡{"md":"As with the discrete-time logistic model that we considered earlier, we can create a bifurcation diagram showing the changes in behavior as we vary the parameter $a$. Here it is:\n\n<img src=\"BifurcationDiagram_Rossler.png\" alt=\"Bifurcation diagram of the Rossler model\" width=\"937\" height=\"580\">\n\nNotice how similar this is to the bifurcation diagram from the discrete-time logistic model. Of course, the precise details are different. But the overall shape or pattern of behavior is exactly the same: we see a sequence of period-doubling bifurcations, occurring closer and closer together, until suddenly chaotic behavior happens. Then, within the region of chaotic behavior, there are small windows of periodic behavior that appear for short parameter intervals, and even within these more period-doubling bifurcations can be seen. Also, within the region of chaotic behavior, some values of the state variable ($J$ in this case) occur more frequently, and others less frequently, resulting in darker or lighter shading. From this, a complex pattern can be seen through the chaos.\n\nIf you're following closely how these bifurcation diagrams are created, you might be wondering how this one can be made, since there are now $3$ state variables in this model instead of $1$, and the trajectory moves smoothly through the state space instead of jumping from one point to another as in a discrete-time model. In this case, we have “discretized” the steady-state behavior of the system by looking at the points where the trajectory intersects the $JT$ plane. We have then decreased these $3$-dimensional points to $1$-dimensional data by looking only at the value of the $J$ variable at each point (since the $T$ values at these points are quite close to $0$ and don't vary much). This is the reason that the vertical axis in the bifurcation diagram above is labeled $J$: it gives the $J$ coordinates of the points at which the trajectory intersects the $JT$ plane. Similar tricks can be employed in other continuous-time models with chaotic behavior.\n\nYou can use the interactive below to explore this model further for the whole range of values of $a$ shown above. With the help of the above bifurcation diagram, see if you can use this interactive to find $a$ values where the limit cycle attractor winds around the $T$ axis $3$ times before forming a loop. What about $6$ times, or $5$ times, or $7$ times?"}︡{"done":true}︡
︠69122975-79af-42b2-94c1-519378d21d1cs︠
@interact
def rossler(a=slider(1, 18, RDF(0.01), default=6, label="$a$")):
    vectorfield(R, J, T) = (-J - T, R + 0.1*J, 0.1 - a*T + R*T)
    t_range = srange(0, 1500, 0.01)
    start = int(900/0.01)
    solution = desolve_odeint(vectorfield, (1, 1, 0), t_range, [R, J, T])
    p = axes3d((R, -25, 25), (J, -25, 25), (T, 0, 40), grid=False)
    p += list_plot(solution[start:], plotjoined=True, color="blue")
    p.show(aspect_ratio=(1, 1, 1), frame=False)
    p2 = list_plot(zip(t_range, solution[:,2])[start:], plotjoined=True)
    p2.show(ymax=40, axes_labels=("$t$", "$T$"))
︡5b2b3d66-3f91-4527-9547-3dd94c146171︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":167,"display_value":true,"label":"$a$","vals":["1.0","1.03","1.06","1.09","1.12","1.15","1.18","1.21","1.24","1.27","1.3","1.33","1.36","1.39","1.42","1.45","1.48","1.51","1.54","1.57","1.6","1.63","1.66","1.69","1.72","1.75","1.78","1.81","1.84","1.87","1.9","1.93","1.96","1.99","2.02","2.05","2.08","2.11","2.14","2.17","2.2","2.23","2.26","2.29","2.32","2.35","2.38","2.41","2.44","2.47","2.5","2.53","2.56","2.59","2.62","2.65","2.68","2.71","2.74","2.77","2.8","2.83","2.86","2.89","2.92","2.95","2.98","3.01","3.04","3.07","3.1","3.13","3.16","3.19","3.22","3.25","3.28","3.31","3.34","3.37","3.4","3.43","3.46","3.49","3.52","3.55","3.58","3.61","3.64","3.67","3.7","3.73","3.76","3.79","3.82","3.85","3.88","3.91","3.94","3.97","4.0","4.03","4.06","4.09","4.12","4.15","4.18","4.21","4.24","4.27","4.3","4.33","4.36","4.39","4.42","4.45","4.48","4.51","4.54","4.57","4.6","4.63","4.66","4.69","4.72","4.75","4.78","4.81","4.84","4.87","4.9","4.93","4.96","4.99","5.02","5.05","5.08","5.11","5.14","5.17","5.2","5.23","5.26","5.29","5.32","5.35","5.38","5.41","5.44","5.47","5.5","5.53","5.56","5.59","5.62","5.65","5.68","5.71","5.74","5.77","5.8","5.83","5.86","5.89","5.92","5.95","5.98","6.01","6.04","6.07","6.1","6.13","6.16","6.19","6.22","6.25","6.28","6.31","6.34","6.37","6.4","6.43","6.46","6.49","6.52","6.55","6.58","6.61","6.64","6.67","6.7","6.73","6.76","6.79","6.82","6.85","6.88","6.91","6.94","6.97","7.0","7.03","7.06","7.09","7.12","7.15","7.18","7.21","7.24","7.27","7.3","7.33","7.36","7.39","7.42","7.45","7.48","7.51","7.54","7.57","7.6","7.63","7.66","7.69","7.72","7.75","7.78","7.81","7.84","7.87","7.9","7.93","7.96","7.99","8.02","8.05","8.08","8.11","8.14","8.17","8.2","8.23","8.26","8.29","8.32","8.35","8.38","8.41","8.44","8.47","8.5","8.53","8.56","8.59","8.62","8.65","8.68","8.71","8.74","8.77","8.8","8.83","8.86","8.89","8.92","8.95","8.98","9.01","9.04","9.07","9.1","9.13","9.16","9.19","9.22","9.25","9.28","9.31","9.34","9.37","9.4","9.43","9.46","9.49","9.52","9.55","9.58","9.61","9.64","9.67","9.7","9.73","9.76","9.79","9.82","9.85","9.88","9.91","9.94","9.97","10.0","10.03","10.06","10.09","10.12","10.15","10.18","10.21","10.24","10.27","10.3","10.33","10.36","10.39","10.42","10.45","10.48","10.51","10.54","10.57","10.6","10.63","10.66","10.69","10.72","10.75","10.78","10.81","10.84","10.87","10.9","10.93","10.96","10.99","11.02","11.05","11.08","11.11","11.14","11.17","11.2","11.23","11.26","11.29","11.32","11.35","11.38","11.41","11.44","11.47","11.5","11.53","11.56","11.59","11.62","11.65","11.68","11.71","11.74","11.77","11.8","11.83","11.86","11.89","11.92","11.95","11.98","12.01","12.04","12.07","12.1","12.13","12.16","12.19","12.22","12.25","12.28","12.31","12.34","12.37","12.4","12.43","12.46","12.49","12.52","12.55","12.58","12.61","12.64","12.67","12.7","12.73","12.76","12.79","12.82","12.85","12.88","12.91","12.94","12.97","13.0","13.03","13.06","13.09","13.12","13.15","13.18","13.21","13.24","13.27","13.3","13.33","13.36","13.39","13.42","13.45","13.48","13.51","13.54","13.57","13.6","13.63","13.66","13.69","13.72","13.75","13.78","13.81","13.84","13.87","13.9","13.93","13.96","13.99","14.02","14.05","14.08","14.11","14.14","14.17","14.2","14.23","14.26","14.29","14.32","14.35","14.38","14.41","14.44","14.47","14.5","14.53","14.56","14.59","14.62","14.65","14.68","14.71","14.74","14.77","14.8","14.83","14.86","14.89","14.92","14.95","14.98","15.01","15.04","15.07","15.1","15.13","15.16","15.19","15.22","15.25","15.28","15.31","15.34","15.37","15.4","15.43","15.46","15.49","15.52","15.55","15.58","15.61","15.64","15.67","15.7","15.73","15.76","15.79","15.82","15.85","15.88","15.91","15.94","15.97","16.0","16.03","16.06","16.09","16.12","16.15","16.18","16.21","16.24","16.27","16.3","16.33","16.36","16.39","16.42","16.45","16.48","16.51","16.54","16.57","16.6","16.63","16.66","16.69","16.72","16.75","16.78","16.81","16.84","16.87","16.9","16.93","16.96","16.99","17.02","17.05","17.08","17.11","17.14","17.17","17.2","17.23","17.26","17.29","17.32","17.35","17.38","17.41","17.44","17.47","17.5","17.53","17.56","17.59","17.62","17.65","17.68","17.71","17.74","17.77","17.8","17.83","17.86","17.89","17.92","17.95","17.98"],"var":"a","width":null}],"flicker":false,"id":"2510fcb2-80d7-47bd-a538-849e86e5e7a9","layout":[[["a",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠238cbda4-1131-4dcc-9123-31494ae680d6is︠
%hide
%md
## The Hastings $3$-species food chain model

Recall the following model from class (and your textbook), in which $Z$ is the population of a top predator such as wolves, $Y$ is the population of a middle species, such as sheep, that is preyed upon by the top predator but is also a predator of the bottom species, $X$, which might be, for example, grass.
$$ \begin{cases}
    X' = r \cdot X \cdot \big( 1 - \frac{X}{k} \big) - m_1 \cdot \frac{X}{h_1 + X} \cdot Y \\
    Y' = b_1 \cdot \big( m_1 \cdot \frac{X}{h_1 + X} \cdot Y \big) - d_1 \cdot Y - m_2 \cdot \frac{Y}{h_2 + Y} \cdot Z \\
    Z' = b_2 \cdot \big( m_2 \cdot \frac{Y}{h_2 + Y} \cdot Z \big) - d_2 \cdot Z
\end{cases} $$

We have seen in class that the parameter values $r = 1$, $k = 1$, $m_1 = 1.6$, $h_1 = 0.3$, $b_1 = 1$, $d_1 = 0.4$, $m_2 = 0.05$, $h_2 = 0.5$, $b_2 = 1$, $d_2 = 0.01$ gives rise to chaotic behavior. We will now look at what happens as we vary the parameter $m_1$, leaving all the other parameters the same.

Below is an interactive that allows you to explore this, followed by the resulting bifurcation diagram. Again, see if you can find values of the parameter that give rise to an ordinary limit cycle attractor, and to a limit cycle attractor that winds around $2$, $3$, or $4$ times before forming a closed loop.

︡900b0973-934d-4b73-b8b9-610a81d8af9d︡{"hide":"input"}︡{"md":"## The Hastings $3$-species food chain model\n\nRecall the following model from class (and your textbook), in which $Z$ is the population of a top predator such as wolves, $Y$ is the population of a middle species, such as sheep, that is preyed upon by the top predator but is also a predator of the bottom species, $X$, which might be, for example, grass.\n$$ \\begin{cases}\n    X' = r \\cdot X \\cdot \\big( 1 - \\frac{X}{k} \\big) - m_1 \\cdot \\frac{X}{h_1 + X} \\cdot Y \\\\\n    Y' = b_1 \\cdot \\big( m_1 \\cdot \\frac{X}{h_1 + X} \\cdot Y \\big) - d_1 \\cdot Y - m_2 \\cdot \\frac{Y}{h_2 + Y} \\cdot Z \\\\\n    Z' = b_2 \\cdot \\big( m_2 \\cdot \\frac{Y}{h_2 + Y} \\cdot Z \\big) - d_2 \\cdot Z\n\\end{cases} $$\n\nWe have seen in class that the parameter values $r = 1$, $k = 1$, $m_1 = 1.6$, $h_1 = 0.3$, $b_1 = 1$, $d_1 = 0.4$, $m_2 = 0.05$, $h_2 = 0.5$, $b_2 = 1$, $d_2 = 0.01$ gives rise to chaotic behavior. We will now look at what happens as we vary the parameter $m_1$, leaving all the other parameters the same. \n\nBelow is an interactive that allows you to explore this, followed by the resulting bifurcation diagram. Again, see if you can find values of the parameter that give rise to an ordinary limit cycle attractor, and to a limit cycle attractor that winds around $2$, $3$, or $4$ times before forming a closed loop. \n\n"}︡{"done":true}︡
︠83c5147e-d185-4ee7-aa49-88911ca7caa4s︠
@interact
def hastings(m1=slider(0.65, 1.8, RDF(0.01), label="$m_1$")):
    vectorfield(X, Y, Z) = (X*(1 - X) - m1*X*Y/(0.3 + X), m1*X*Y/(0.3 + X) - 0.4*Y - 0.05*Y*Z/(0.5 + Y), 0.05*Y*Z/(0.5 + Y) - 0.01*Z)
    initial_state = (12.5*sqrt(0.002704 - 0.0008*m1) + 0.4, 0.1, 12.5*m1 + 375*sqrt(0.002704 - 0.0008*m1) - 24.4)
    t_range = srange(0, 5000, 0.1)
    start = int(1000/0.1)
    solution = desolve_odeint(vectorfield, initial_state, t_range, [X, Y, Z])
    p = axes3d((X, 0, 1), (Y, 0, 0.5), (Z, 0, 11), grid=False, origin=(0, 0, 0))
    p += list_plot(solution[start:], plotjoined=True)
    p.show(aspect_ratio=(1, 2, 0.25), frame=False)
︡6475be4d-7716-4938-8fb9-b598db79c553︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"$m_1$","vals":["0.650000000000000","0.660000000000000","0.670000000000000","0.680000000000000","0.690000000000000","0.700000000000000","0.710000000000000","0.720000000000000","0.730000000000000","0.740000000000000","0.750000000000000","0.760000000000000","0.770000000000000","0.780000000000000","0.790000000000000","0.800000000000000","0.810000000000000","0.820000000000000","0.830000000000000","0.840000000000000","0.850000000000000","0.860000000000000","0.870000000000000","0.880000000000000","0.890000000000000","0.900000000000000","0.910000000000000","0.920000000000000","0.930000000000000","0.940000000000000","0.950000000000000","0.960000000000000","0.970000000000000","0.980000000000000","0.990000000000000","1.00000000000000","1.01000000000000","1.02000000000000","1.03000000000000","1.04000000000000","1.05000000000000","1.06000000000000","1.07000000000000","1.08000000000000","1.09000000000000","1.10000000000000","1.11000000000000","1.12000000000000","1.13000000000000","1.14000000000000","1.15000000000000","1.16000000000000","1.17000000000000","1.18000000000000","1.19000000000000","1.20000000000000","1.21000000000000","1.22000000000000","1.23000000000000","1.24000000000000","1.25000000000000","1.26000000000000","1.27000000000000","1.28000000000000","1.29000000000000","1.30000000000000","1.31000000000000","1.32000000000000","1.33000000000000","1.34000000000000","1.35000000000000","1.36000000000000","1.37000000000000","1.38000000000000","1.39000000000000","1.40000000000000","1.41000000000000","1.42000000000000","1.43000000000000","1.44000000000000","1.45000000000000","1.46000000000000","1.47000000000000","1.48000000000000","1.49000000000000","1.50000000000000","1.51000000000000","1.52000000000000","1.53000000000000","1.54000000000000","1.55000000000000","1.56000000000000","1.57000000000000","1.58000000000000","1.59000000000000","1.60000000000000","1.61000000000000","1.62000000000000","1.63000000000000","1.64000000000000","1.65000000000000","1.66000000000000","1.67000000000000","1.68000000000000","1.69000000000000","1.70000000000000","1.71000000000000","1.72000000000000","1.73000000000000","1.74000000000000","1.75000000000000","1.76000000000000","1.77000000000000","1.78000000000000","1.79000000000000","1.80000000000000"],"var":"m1","width":null}],"flicker":false,"id":"da03a64c-6699-4843-b64e-4969097a7e15","layout":[[["m1",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠1d3ce5dc-ffb1-4fb4-bb30-14afb03363aais︠
%hide
%md

<img src="BifurcationDiagram_Hastings.png" alt="Bifurcation diagram of the Hasting 3-species food chain model" width="938" height="581">

Bifurcation diagram for the Hastings model (Note: The parameter is called $b_1$ in the diagram, but it corresponds to what we've called $m_1$ above.)

It is also possible to generate a bifurcation diagram like this for the Lorenz model (which was covered in your lab as “Romeo, Juliet, and Juliet's Nurse”). Once again, it will have most of the exact same features as all of the bifurcation diagrams we have looked at above. The precise shape will be different, but in terms of its overall look, you could say all of these look “about the same”.

︡6e6fe9be-bfc4-4b9a-9c82-58f56e4c4b22︡{"hide":"input"}︡{"md":"\n<img src=\"BifurcationDiagram_Hastings.png\" alt=\"Bifurcation diagram of the Hasting 3-species food chain model\" width=\"938\" height=\"581\">\n\nBifurcation diagram for the Hastings model (Note: The parameter is called $b_1$ in the diagram, but it corresponds to what we've called $m_1$ above.)\n\nIt is also possible to generate a bifurcation diagram like this for the Lorenz model (which was covered in your lab as “Romeo, Juliet, and Juliet's Nurse”). Once again, it will have most of the exact same features as all of the bifurcation diagrams we have looked at above. The precise shape will be different, but in terms of its overall look, you could say all of these look “about the same”."}︡{"done":true}︡
︠d67dfe49-9d81-45e2-bae0-1aaf86955f7ais︠
%hide
%md
## Summary: What these bifurcation diagrams mean

In brief, a bifurcation diagram like the one above means that as we increase the parameter $r$, the steady-state behavior of the system changes in the following way:

$$ \text{equilibrium} \to \text{oscillations} \to \text{complex oscillations} \to \text{chaos} $$

Here, “complex oscillations” refer to the oscillatory behavior with higher periods, after one or more period-doubling bifurcations have occurred.

Recall that the change from **equilibrium** to **oscillations** is a *Hopf bifurcation*, and the change from regular **oscillations** to **complex oscillations** is a *period-doubling bifurcation*. Also, within the region of complex oscillations, infinitely many more period-doubling bifurcations occur, closer and closer together. This is sometimes referred to as a “period-doubling cascade”, which then leads to chaotic behavior. In general, the above pattern is often referred to as the “period-doubling route to chaos”.

︡9a035cb9-f1f7-4690-af9d-da3f92d2fadf︡{"hide":"input"}︡{"md":"## Summary: What these bifurcation diagrams mean\n\nIn brief, a bifurcation diagram like the one above means that as we increase the parameter $r$, the steady-state behavior of the system changes in the following way:\n\n$$ \\text{equilibrium} \\to \\text{oscillations} \\to \\text{complex oscillations} \\to \\text{chaos} $$\n\nHere, “complex oscillations” refer to the oscillatory behavior with higher periods, after one or more period-doubling bifurcations have occurred.\n\nRecall that the change from **equilibrium** to **oscillations** is a *Hopf bifurcation*, and the change from regular **oscillations** to **complex oscillations** is a *period-doubling bifurcation*. Also, within the region of complex oscillations, infinitely many more period-doubling bifurcations occur, closer and closer together. This is sometimes referred to as a “period-doubling cascade”, which then leads to chaotic behavior. In general, the above pattern is often referred to as the “period-doubling route to chaos”."}︡{"done":true}︡
︠0d951757-d6ad-43f6-83c1-b8dd21a3c15c︠









